#!/usr/bin/perl
# vim: sw=4 ts=4
#
# Written by Marco d'Itri <md@linux.it> and released to the public domain.
#
# The program expects on input a list of prefixes, optionally followed by
# a comment, one per line.
#
# It expects to find the ARIN database in /tmp/networks.csv:
#   wget -P /tmp/ ftp://ftp.arin.net/pub/resource_registry_service/networks.csv

use v5.36;
use autodie;

use Text::CSV;			# libtext-csv-perl
use Net::CIDR ':all';	# libnet-cidr-perl
use Net::Patricia;		# libnet-patricia-perl

my %First_AS_Category = (
	293		=> 'USEDU',	#'ESNET',
#	57463	=> 'NETIX',
	32787	=> 'SCRUB', # (former Prolexic)
	19551	=> 'SCRUB', # Imperva
	35280	=> 'SCRUB', # F5
);

my %Second_AS_Category = (
	293		=> 'USEDU',	#'ESNET',
	668		=> 'USMIL',
	2149	=> 'DNSROOT',
	21556	=> 'DNSROOT',
);

##############################################################################
# returns a Net::Patricia object containing the networks marked as legacy
# The data associated to each prefix is an array reference containing:
# 0: network, 1: organization name
sub read_networks ($file) {
	open(my $fh, '<', $file);
	my $csv = Text::CSV->new({
		binary		=> 1,
		sep_char	=> ',',
	});

	my $header = $csv->getline($fh);
	die if not $header->[5] eq 'Status';

	my $pt = Net::Patricia->new;
	while (my $row = $csv->getline($fh)) {
		next if not $row->[5] eq 'Basic Legacy Services';

		# convert the range in columns 4 and 5 to CIDR prefixes
		my (@cidr) = map {
			s/(^|\.)0+(?=[0-9])/$1/g;	# clean up the leading zeros
			$_;
		} range2cidr("$row->[3]-$row->[4]");

		# beware: a single range may be expanded to multiple CIDR prefixes
		foreach (@cidr) {
			eval { $pt->add_string($_, [$_, $row->[0]]); };
			die "$_: $@" if $@;
		}
	}

	close $fh;
	return $pt;
}

##############################################################################
{
	my $pt = read_networks('/tmp/networks-legacy.csv');
	my (%category, %category_net, %firstas);

	foreach (<<>>) {
		chomp;
		next if /^\s*$/;
		my ($prefix, $comment) = split(/\s+/, $_, 2);
		$comment //= '';
		# remove the netmask, because some announced networks are listed as
		# multiple adjacent prefixes in the ARIN file
		my $ip = $prefix;
		$ip =~ s#/.+##;
		my $found = $pt->match_string($ip) or next;
		my ($cidr, $org) = @$found;

		my ($firstas, $secondas) = $comment =~ /^(\d+)\b(?:\s+(\d+)\b)?/;
		$firstas{$1}++ if $firstas;
		$secondas //= '';

		my $category = '';

		if (exists $First_AS_Category{$firstas}) {
			$category = $First_AS_Category{$firstas};
		} elsif (exists $Second_AS_Category{$secondas}) {
			$category = $Second_AS_Category{$secondas};
		} elsif ($org =~ /^Amateur Radio Digital Communications/) {
			$category = 'AMPR';
		} elsif ($org =~ /National Lab(s|oratory|oratories)/) {
			$category = 'USEDU', #'USLAB';
		} elsif ($org =~ /University|College|School|Laboratory|Library/) {
			$category = 'USEDU';
		} elsif ($org =~ /^(?:City of|State of|(?:\S+ )?(?:Dept\.?|Department) of|.+ County |.+ Police |National Aeronautics)/) {
			$category = 'USGOV';
		} elsif ($org =~ /^(?:PSINet|AGIS|Performance Systems International)/) {
			$category = 'COGENT';
		} else {
			$category = 'OTHER';
		}
		$category{$category}++;
		#		push($category_net{$category}, $cidr_len);

		#say "$prefix\t$category\t$comment\t$cidr\t$org";
		say "$prefix\t$category\t$comment\t$org";
	}

	say '';
	say "$_\t$category{$_}" foreach
		reverse sort { $category{$a} <=> $category{$b} } keys %category;

	say '';
	say "$_\t$firstas{$_}" foreach
		grep { $firstas{$_} > 10 }
		reverse sort { $firstas{$a} <=> $firstas{$b} } keys %firstas;
}

